@@ -640,6 +640,21 @@ class Fastq
     Bio::Sequence.adapter(self, Bio::Sequence::Adapter::Fastq)
   end
 
+  # Masks low quality sequence regions.
+  # For each sequence position, if the quality score is smaller than
+  # the threshold, the sequence in the position is replaced with
+  # <em>mask_char</em>.
+  #
+  # Note: This method does not care quality_score_type.
+  # ---
+  # *Arguments*:
+  # * (required) <em>threshold</em> : (Numeric) threshold
+  # * (optional) <em>mask_char</em> : (String) character used for masking
+  # *Returns*:: Bio::Sequence object
+  def mask(threshold, mask_char = 'n')
+    to_biosequence.mask_with_quality_score(threshold, mask_char)
+  end
+
 end #class Fastq
 
 end #module Bio
