@@ -179,6 +179,28 @@ class Location
     return 0
   end
 
+  # If _other_ is equal with the self, returns true.
+  # Otherwise, returns false.
+  # ---
+  # *Arguments*:
+  # * (required) _other_: any object
+  # *Returns*:: true or false
+  def ==(other)
+    return true if super(other)
+    return false unless other.instance_of?(self.class)
+    flag = false
+    [ :from, :to, :strand, :sequence, :lt, :gt,
+      :xref_id, :carat ].each do |m|
+      begin
+        flag = (self.__send__(m) == other.__send__(m))
+      rescue NoMethodError, ArgumentError, NameError
+        flag = false
+      end
+      break unless flag
+    end
+    flag
+  end
+
 end # Location
 
 # == Description
@@ -350,6 +372,23 @@ class Locations
     end
   end
 
+  # If _other_ is equal with the self, returns true.
+  # Otherwise, returns false.
+  # ---
+  # *Arguments*:
+  # * (required) _other_: any object
+  # *Returns*:: true or false
+  def ==(other)
+    return true if super(other)
+    return false unless other.instance_of?(self.class)
+    if self.locations == other.locations and
+        self.operator == other.operator then
+      true
+    else
+      false
+    end
+  end
+
   # Iterates on each Bio::Location object.
   def each
     @locations.each do |x|
