@@ -5,13 +5,15 @@
 #               Mitsuteru C. Nakao <n@bioruby.org>
 # License::     The Ruby License
 #
-# $Id: test_na.rb,v 1.5 2007/04/05 23:35:44 trevor Exp $
+# $Id:$
 #
 
+# loading helper routine for testing bioruby
 require 'pathname'
-libpath = Pathname.new(File.join(File.dirname(__FILE__), ['..'] * 4, 'lib')).cleanpath.to_s
-$:.unshift(libpath) unless $:.include?(libpath)
+load Pathname.new(File.join(File.dirname(__FILE__), ['..'] * 3,
+                            'bioruby_test_helper.rb')).cleanpath.to_s
 
+# libraries needed for the tests
 require 'test/unit'
 require 'bio/sequence'
 require 'bio/sequence/na'  
@@ -175,6 +177,68 @@ module Bio
 
   end
 
+  class TestSequenceNACommon < Test::Unit::TestCase
+
+    def setup
+      @obj  = Bio::Sequence::NA.new('atgcatgcatgcatgcaaaa')
+    end
+
+    def test_to_s
+      assert_equal('atgcatgcatgcatgcaaaa', @obj.to_s)
+    end
+
+    def test_to_str
+      assert_equal('atgcatgcatgcatgcaaaa', @obj.to_str)
+    end
+
+    def test_seq
+      str = "atgcatgcatgcatgcaaaa"
+      assert_equal(str, @obj.seq)
+    end
+
+    # <<(*arg)
+    def test_push
+      str = "atgcatgcatgcatgcaaaaa"
+      assert_equal(str, @obj << "A")
+    end
+
+    # concat(*arg)
+    def test_concat
+      str = "atgcatgcatgcatgcaaaaa"
+      assert_equal(str, @obj.concat("A"))
+    end
+
+    # +(*arg)
+    def test_sum 
+      str = "atgcatgcatgcatgcaaaaatgcatgcatgcatgcaaaa"
+      assert_equal(str, @obj + @obj)
+    end
+
+    # window_search(window_size, step_size = 1)
+    def test_window_search
+      @obj.window_search(4) do |subseq|
+        assert_equal(20, @obj.size)
+      end
+    end
+
+    #total(hash)
+    def test_total
+      hash = {'a' => 1, 'c' => 2, 'g' => 4, 't' => 3}
+      assert_equal(44.0, @obj.total(hash))
+    end
+
+    def test_composition
+      composition = {"a"=>8, "c"=>4, "g"=>4, "t"=>4}
+      assert_equal(composition, @obj.composition)
+    end
+    
+    def test_splicing
+      #(position)
+      assert_equal("atgcatgc", @obj.splicing("join(1..4, 13..16)"))
+    end
+  end
+
+
   class TestSequenceNATranslation < Test::Unit::TestCase
     def setup
 
