@@ -1,19 +1,22 @@
 #
-# = test/bio/tc_pathway.rb - Unit test for Bio::Pathway
+# = test/unit/bio/test_reference.rb - Unit test for Bio::Reference
 #
 # Copyright::  Copyright (C) 2006 
 #              Mitsuteru C. Nakao <n@bioruby.org>
 # License::    The Ruby License
 #
-# $Id: test_reference.rb,v 1.3 2007/04/05 23:35:42 trevor Exp $
+# $Id:$
 #
 
+# loading helper routine for testing bioruby
 require 'pathname'
-libpath = Pathname.new(File.join(File.dirname(__FILE__), [".."] * 3, "lib")).cleanpath.to_s
-$:.unshift(libpath) unless $:.include?(libpath)
+load Pathname.new(File.join(File.dirname(__FILE__), ['..'] * 2,
+                            'bioruby_test_helper.rb')).cleanpath.to_s
 
+# libraries needed for the tests
 require 'test/unit'
 require 'bio/reference'
+require 'bio/compat/references'
 
 
 module Bio
@@ -90,7 +93,7 @@ module Bio
     end
 
     def test_format_endnote
-      str = "%0 Journal Article\n%A Hoge, J.P.\n%A Fuga, F.B.\n%D 2001\n%T Title of the study.\n%J Theor. J. Hoge\n%V 12\n%N 3\n%P 123-145\n%M 12345678\n%U http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=Retrieve&db=PubMed&dopt=Citation&list_uids=12345678\n%X Hoge fuga. hoge fuga.\n%K Hoge\n%+ Tokyo"
+      str = "%0 Journal Article\n%A Hoge, J.P.\n%A Fuga, F.B.\n%D 2001\n%T Title of the study.\n%J Theor. J. Hoge\n%V 12\n%N 3\n%P 123-145\n%M 12345678\n%U http://example.com\n%X Hoge fuga. hoge fuga.\n%K Hoge\n%+ Tokyo"
       assert_equal(str, @obj.format('endnote'))
       assert_equal(str, @obj.endnote)
     end
@@ -102,21 +105,47 @@ module Bio
     end
 
     def test_format_bibtex
-      str =<<END
-        @article{PMID:12345678,
-          author  = {Hoge, J.P. and Fuga, F.B.},
-          title   = {Title of the study.},
-          journal = {Theor. J. Hoge},
-          year    = {2001},
-          volume  = {12},
-          number  = {3},
-          pages   = {123--145},
-        }
-END
+      str =<<__END__
+@article{PMID:12345678,
+  author       = {Hoge, J.P. and Fuga, F.B.},
+  title        = {Title of the study.},
+  journal      = {Theor. J. Hoge},
+  year         = {2001},
+  volume       = {12},
+  number       = {3},
+  pages        = {123--145},
+  url          = {http://example.com},
+}
+__END__
       assert_equal(str, @obj.format('bibtex'))
       assert_equal(str, @obj.bibtex)
     end
 
+    def test_format_bibtex_with_arguments
+      str =<<__END__
+@inproceedings{YourArticle,
+  author       = {Hoge, J.P. and Fuga, F.B.},
+  title        = {Title of the study.},
+  year         = {2001},
+  volume       = {12},
+  number       = {3},
+  pages        = {123--145},
+  booktitle    = {Theor. J. Hoge},
+  month        = {December},
+}
+__END__
+      assert_equal(str, @obj.format('bibtex', 'inproceedings', 'YourArticle',
+                                    { 'journal'   => false,
+                                      'url' => false,
+                                      'booktitle' => @obj.journal,
+                                      'month' => 'December'}))
+      assert_equal(str, @obj.bibtex('inproceedings', 'YourArticle',
+                                    { 'journal'   => false,
+                                      'url' => false,
+                                      'booktitle' => @obj.journal,
+                                      'month' => 'December'}))
+    end
+
     def test_format_rd
       str = "== Title of the study.\n\n* Hoge, J.P. and Fuga, F.B.\n\n* Theor. J. Hoge 2001 12:123-145 [PMID:12345678]\n\nHoge fuga. hoge fuga."
       assert_equal(str, @obj.format('rd'))
@@ -174,13 +203,40 @@ END
 
   class TestReferences < Test::Unit::TestCase
 
+    class NullStderr
+      def initialize
+        @log = []
+      end
+
+      def write(*arg)
+        #p arg
+        @log.push([ :write, *arg ])
+        nil
+      end
+
+      def method_missing(*arg)
+        #p arg
+        @log.push arg
+        nil
+      end
+    end #class NullStderr
+
     def setup
+      # To suppress warning messages, $stderr is replaced by dummy object.
+      @stderr_orig = $stderr
+      $stderr = NullStderr.new
+
       hash = {}
       ary = [Bio::Reference.new(hash),
              Bio::Reference.new(hash)]
       @obj = Bio::References.new(ary)
     end
 
+    def teardown
+      # bring back $stderr
+      $stderr = @stderr_orig
+    end
+
     def test_append
       hash = {}
       ref = Bio::Reference.new(hash)
